from photospicker.picker.abstract_picker import AbstractPicker
import random


class RandomPicker(AbstractPicker):
    """Pick photos randomly"""

    def scan(self):
        """Scan the given path for building picked file paths list"""
        self._notify_progress(0)
        random.shuffle(self._files_to_scan)
        self._notify_progress(len(self._files_to_scan))
        self._notify_end()

        self._picked_file_paths = self._files_to_scan[0:self._photos_count]



from photospicker.filter.abstract_filter import AbstractFilter
from PIL import Image


class ResizeFilter(AbstractFilter):
    """Resize a photo"""

    def __init__(self, max_width, max_height):
        """
        Constructor

        :param int max_width : max width after filter execution
        :param int max_height: max height after filter execution
        """
        self._width = max_width
        self._height = max_height

    def execute(self, original_img, exif_data):
        """
        Resize photo

        :param Image original_img: image object to resize
        :param dict exif_data    : image exif data

        :return Image
        """
        (original_width, original_height) = original_img.size

        diff_wh = self._width - self._height
        sign_diff_wh = diff_wh / abs(diff_wh)
        diff_original_wh = original_width - original_height
        sign_diff_original_wh = diff_original_wh / abs(diff_original_wh)

        if sign_diff_wh == sign_diff_original_wh:
            w = self._width
            h = self._height
        else:
            w = self._height
            h = self._width

        if original_width / original_height > w / h:
            ratio = float(w) / original_width
        else:
            ratio = float(h) / original_height

        resized_img = original_img.resize(
            (int(ratio * original_width), int(ratio * original_height)),
            Image.ANTIALIAS
        )

        return resized_img



from exceptions import BaseException


class PickerException(BaseException):
    """Exception when picking photos"""

    # Error constants
    NOT_FOUND = 1
    NOT_EMPTY = 2

    def __init__(self, code, message):
        """
        Constructor

        :param int code   : error code
        :param str message: error message
        """
        self._code = code
        self._message = message

    @property
    def code(self):  # pragma no cover
        """
        Getter code

        :return int
        """
        return self._code

    @property
    def message(self):  # pragma no cover
        """
        Getter message

        :return str
        """
        return self._message



from photospicker.uploader.abstract_uploader import AbstractUploader
from photospicker.exception.picker_exception import PickerException
import os


class FilesystemUploader(AbstractUploader):
    """Copy picked photo to a filesystem empty directory"""

    def __init__(self, folder_path):
        """
        Constructor

        :param str folder_path: target folder path in filesystem
        """
        super(FilesystemUploader, self).__init__()
        self._path = folder_path

    def initialize(self):  # pragma: no cover
        """Check target directory"""
        if not os.path.isdir(self._path):
            raise PickerException(
                PickerException.NOT_FOUND,
                "Directory {path} not found".format(path=self._path)
            )

        if os.listdir(self._path):
            raise PickerException(
                PickerException.NOT_EMPTY,
                "Directory {path} not empty".format(path=self._path)
            )

    def upload(self, binary, original_filename):
        """
        Upload or copy files to destination

        :param str binary           : binary data to upload
        :param str original_filename: original file name
        """
        filename = self._build_filename(original_filename)
        with open(os.path.join(self._path, filename), 'w') as f:
            f.write(binary)



from event.start_upload_event import StartUploadEvent
from event.end_upload_event import EndUploadEvent
from event.start_filter_event import StartFilterEvent
from event.end_filter_event import EndFilterEvent
from zope.event import notify
from PIL import Image
from io import BytesIO
from photospicker.picker.abstract_picker import AbstractPicker  # noqa
from photospicker.uploader.abstract_uploader import AbstractUploader  # noqa
import ntpath


class PhotosPicker:
    """
    Select photos accorting to a chosen strategy and
    copy them to a chosen destination
    """

    def __init__(self, picker, filters, uploader):
        """
        Constructor

        :param AbstractPicker picker    : photo selection strategy
        :param tuple filters            : filters
        :param AbstractUploader uploader: upload strategy
        """
        self._picker = picker
        self._filters = filters
        self._uploader = uploader

    def run(self):
        """Run photo selection and upload"""
        self._picker.initialize()
        self._picker.scan()

        total_picked = len(self._picker.picked_file_paths)

        self._uploader.initialize()
        for key, filepath in enumerate(self._picker.picked_file_paths):
            rank = key + 1

            # If no filter is applyed, save without quality loss
            if len(self._filters) == 0:
                with open(filepath, mode='rb') as f:
                    file_content = f.read()
            else:
                original_img = Image.open(filepath)
                exif_data = original_img._getexif()
                img = original_img.copy()

                for photo_filter in self._filters:
                    notify(StartFilterEvent(photo_filter, filepath))
                    img = photo_filter.execute(img, exif_data)
                    notify(EndFilterEvent(photo_filter, filepath))

                b = BytesIO()
                img.save(b, original_img.format)
                file_content = b.getvalue()

            notify(StartUploadEvent(filepath, rank, total_picked))
            self._uploader.increase_photo_counter()
            self._uploader.upload(file_content, ntpath.basename(filepath))
            notify(EndUploadEvent(filepath, rank, total_picked))



from abc import ABCMeta, abstractmethod
from photospicker.event.scan_progress_event import ScanProgressEvent
from zope import event
import os
import fnmatch


class AbstractPicker:
    """
    Abstract class for creating "picker" classes

    A picker object select files in a path according to a strategy
    which characterizes the picker
    """

    __metaclass__ = ABCMeta

    def __init__(self, directory_path, photos_count, patterns=None):
        """
        Constructor

        :param str  directory_path: directory to scan
        :param int  photos_count:   photos count to pick
        :param list patterns:       patterns (in lowercase) that files must
                                    match for being scanned
        :raise TypeError
        """
        self._root = directory_path
        self._files_to_scan = []
        self._picked_file_paths = []
        self._photos_count = photos_count

        if patterns is None:
            patterns = ['*.tif', '*.tiff', '*.jpg', '*.jpeg', '*.png']
        elif not isinstance(patterns, list):
            raise TypeError("patterns argument must be a list")

        self._patterns = patterns

    @property
    def picked_file_paths(self):
        """Return an array of the picked file paths"""
        return self._picked_file_paths

    def initialize(self):
        """Fill in the list of files to scan"""
        for root, dirnames, filenames in os.walk(self._root):
            for filename in filenames:
                for pattern in self._patterns:
                    if fnmatch.fnmatch(filename.lower(), pattern):
                        self._files_to_scan.append(os.path.join(
                            root,
                            filename
                        ))

    @abstractmethod
    def scan(self):  # pragma: no cover
        """
        Scan the given path for building picked file paths list

        :raise NotImplementedError
        """
        raise NotImplementedError()

    def _notify_progress(self, scanned):
        """
        Notify the progress state of the scan

        :param int scanned: scanned files count
        """
        event.notify(ScanProgressEvent(
            scanned,
            len(self._files_to_scan),
            False
        ))

    def _notify_end(self):
        """Notify the end of the scan"""
        to_scan = len(self._files_to_scan)
        event.notify(ScanProgressEvent(to_scan, to_scan, True))



from photospicker.filter.abstract_filter import AbstractFilter
from PIL import ExifTags


class RotateFilter(AbstractFilter):

    def __init__(self, expand=True):
        """
        Constructor

        :param bool expand: wheteher the image will be
            expanded to hold the entire rotated image
        """
        self._expand = expand

    def execute(self, original_img, exif_data):
        """
        Rotate photo according to EXIF data

        :param Image original_img: image object to rotate
        :param dict exif_data    : image exif data

        :return Image
        """
        orientation_key = None
        for key in ExifTags.TAGS.keys():
            if ExifTags.TAGS.get(key) == 'Orientation':
                orientation_key = key

        angle = None
        if exif_data:
            if orientation_key is not None and orientation_key in exif_data:
                if exif_data[orientation_key] == 3:
                    angle = 180
                elif exif_data[orientation_key] == 6:
                    angle = 270
                elif exif_data[orientation_key] == 8:
                    angle = 90

        if angle is None:
            return original_img

        return original_img.rotate(angle, expand=self._expand)



import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='photos-picker',
    version='0.1.2',
    description='Pick photos following a given strategy and upload them to various destinations',
    author='Laurent VOULLEMIER',
    author_email='laurent.voullemier@gmail.com',
    url='https://github.com/l-vo/photos-picker',
    packages=find_packages(),
    install_requires=['Pillow', 'zope.event', 'dropbox'],
    include_package_data=True,
    zip_safe=False,
    long_description=read('README.md'),
    license='MIT',
    keywords='photos upload photoframe',
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2'
    ]
)



class StartUploadEvent(object):
    """Event dispatched when an upload starts"""

    def __init__(self, filepath, upload_file_rank, files_to_upload):
        """
        Constructor

        :param str filepath        : file which starts to be uploaded
        :param int upload_file_rank: rank of the file in upload queue
        :param int files_to_upload : total files to upload count
        """
        self._filepath = filepath
        self._upload_file_rank = upload_file_rank
        self._files_to_upload = files_to_upload

    @property
    def filepath(self):  # pragma: no cover
        """
        Getter for the file which starts to be uploaded

        :return str
        """
        return self._filepath

    @property
    def upload_file_rank(self):  # pragma: no cover
        """
        Getter for the rank of the file in upload queue

        :return int
        """
        return self._upload_file_rank

    @property
    def files_to_upload(self):  # pragma: no cover
        """
        Getter for the total files to upload count

        :return int
        """
        return self._files_to_upload
